@@ -4,7 +4,7 @@ import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 from .config import Style
-from .utils import PickleUtils, Rosetta, make_dict
+from .utils import PickleUtils, Rosetta, make_dict, get_num_workers
 from .data_quality import basis_quality_checks, basis_quality_plots, fill_ratio
 from copy import deepcopy
 import statsmodels.api as sm
@@ -12,8 +12,7 @@ from .plotting import visualize_x_y, visualize_1d_distribution, visualize_surfac
 import pathlib
 from pydantic import BaseModel
 from sklearn.feature_selection import SelectKBest, chi2
-from multiprocessing import Pool, cpu_count
-from tqdm.auto import tqdm
+from multiprocessing import Pool
 
 
 class SVD(BaseModel):
@@ -123,21 +122,31 @@ class VectorMultiset(PickleUtils, Style, Rosetta):
     # Visualizations
     ################
 
-    def visualize_x_y(self, x_name: str, y_name: str, cumulative: Union[str, List[str]] = '', **kwargs) -> plt.Figure:
+    def visualize_x_y(self, x: Union[str, List[Any], Any], y: Union[str, List[Any], Any],
+                      cumulative: Union[str, List[str]] = '', **kwargs) -> plt.Figure:
         """ Visualize in two dimensions
 
-        :param x_name: name of the x-axis data feature
-        :param y_name: name of the y-axis data feature
+        :param x: name of the x-axis data feature (or the data itself)
+        :param y: name of the y-axis data feature (or the data itself)
         :param cumulative: Which (if any) dimensions should be cumulative, e.g. 'x' or ['x','y'] or 'xy'
         :param kwargs: additional kwargs for isthmuslib.visualize_x_y, passed through to matplotlib.pyplot.scatter()
         :return: figure handle for the plot
         """
-        x_data: List[Any] = self.values(x_name, cumulative='x' in cumulative)
-        y_data: List[Any] = self.values(y_name, cumulative='y' in cumulative)
-        kwargs.setdefault('xlabel', self.translate(x_name))
-        kwargs.setdefault('ylabel', self.translate(y_name))
+        if isinstance(x, str):
+            x_data: List[Any] = self.values(x, cumulative='x' in cumulative)
+            kwargs.setdefault('xlabel', self.translate(x))
+        else:
+            x_data: Any = x
+
+        if isinstance(y, str):
+            y_data: List[Any] = self.values(y, cumulative='y' in cumulative)
+            kwargs.setdefault('ylabel', self.translate(x))
+        else:
+            y_data: Any = y
+
         kwargs.setdefault('title', self.translate(self.name_root))
-        return visualize_x_y(x_data, y_data, style=Style(**self.dict()), **kwargs)
+        style: Style = kwargs.pop('style', Style(**self.dict()))
+        return visualize_x_y(x_data, y_data, style=style, **kwargs)
 
     def scatter(self, *args, **kwargs) -> plt.Figure:
         """ Creates a 2D scatter plot of x and y data (wraps visualize_x_y) """
@@ -348,9 +357,9 @@ class VectorSequence(VectorMultiset):
     basis_col_name: str = 'basis'
     error_if_basis_quality_issues: bool = False
 
-    def __init__(self, **data: Any):
+    def __init__(self, skip_vector_sequence_init: bool = False, **data: Any):
         super().__init__(**data)
-        if self.data is not None:
+        if (self.data is not None) and (not skip_vector_sequence_init):
             self.sort(by=self.basis_col_name, inplace=True, reset_index=True)
             if self.error_if_basis_quality_issues:
                 is_ok, explanation = self.passes_basis_quality_checks()
@@ -456,7 +465,8 @@ class VectorSequence(VectorMultiset):
                                                  inplace=False, reset_index=True)
         return {**function(window_data, *args, **kwargs), 'window_start': start_at, 'window_width': window_width}
 
-    def sliding_window(self, function: Callable[[Any, List[Any], Dict[str, Any]], Dict[str, Any]],
+    #                                       vv sequence vv       vv args vv          vv kwargs vv
+    def sliding_window(self, function: Callable[[Any, Union[Tuple[Any], List[Any]], Dict[str, Any]], Dict[str, Any]],
                        window_widths: List[float] = None, window_starts: List[Any] = None, step_size: float = None,
                        parallelize_sliding_window: Union[bool, int] = True, *args, **kwargs) -> SlidingWindowResults:
         """ Apply function in a sliding window over the sequence
@@ -493,30 +503,22 @@ class VectorSequence(VectorMultiset):
                     window_starts: List[float] = list(np.arange(min(basis), max(basis) - window_width, window_width))
             list_of_start_and_width_tuples += [(start_time, window_width) for start_time in window_starts]
 
-        # If evaluating in parallel, use at most the number of CPUs (and use all of them if count not specified)
-        if parallelize_sliding_window and (cpu_count() > 1):
-            if isinstance(parallelize_sliding_window, bool):
-                num_workers: int = cpu_count()
-            else:
-                num_workers: int = min(cpu_count(), parallelize_sliding_window)
+        # If parallelize_sliding_window != False, run in parallel using starmap() from multiprocessing library `Pool`
+        if parallelize_sliding_window:
+            num_workers: int = get_num_workers(parallelize_sliding_window)
 
+            # Create a Pool and process the evaluations in parallel
             with Pool(num_workers) as pool:
                 evaluations: List[Dict[Any, Any]] = pool.starmap(
                     func=self.evaluate_over_window,
                     iterable=[(function, start, width, args, kwargs) for start, width in list_of_start_and_width_tuples]
                 )
 
-        # If evaluating in serial, it's simply a `for` loop
+        # If parallelize_sliding_window == False (or None) use `for` loop in list comprehension -> serial processing
         else:
-            evaluations: List[Dict[Any, Any]] = []
-            for start, width in list_of_start_and_width_tuples:
-                evaluations.append(self.evaluate_over_window(
-                    function=function,
-                    start_at=start,
-                    window_width=width,
-                    args=args,
-                    kwargs=kwargs
-                ))
+            evaluations: List[Dict[Any, Any]] = [self.evaluate_over_window(
+                function=function, start_at=start, window_width=width, args=args, kwargs=kwargs
+            ) for start, width in list_of_start_and_width_tuples]
 
         return SlidingWindowResults(window_width_col_name='window_width', window_start_col_name='window_start',
                                     data=pd.DataFrame(evaluations), name_root=self.name_root)
@@ -691,154 +693,6 @@ class VectorSequence(VectorMultiset):
 ################
 
 
-def auto_extract_from_text(input_string: str, return_type: str = 'dataframe',
-                           left: str = '[<<', key_value_delimiter: str = '=', right: str = '>>]',
-                           basis_col_name: str = None, record_delimiter: str = '[@@@]',
-                           disable_progress_bar: bool = None) -> Union[pd.DataFrame, VectorSequence, VectorMultiset]:
-    """
-    Extracts a data frame from a string
-
-    Input: "the [@@@] quick [<<x='foo'>>] brown[<<y=9>>] [@@@]fox [<<y=93>>]"
-    Output: (data frame)
-                   x   y
-            0  'foo'   9
-            1    NaN  93
-
-    Hint, write your logs using python fstrings like f"Foo [<<{x=}>>] and [<<{y=}>>]"
-
-    :param input_string: string to be parsed
-    :param record_delimiter: substring between rows
-    :param left: left side of a record
-    :param key_value_delimiter: marker between the variable name and its value
-    :param right: right side of a record
-    :param basis_col_name: optional, specify a basis column name for a VectorSequence (otherwise ignored)
-    :param return_type: what format to return the data in
-    :param disable_progress_bar: pass anything True to silence the progress bar
-    :return: pandas.DataFrame or VectorMultiset or VectorSequence
-    """
-    df_output: pd.DataFrame = pd.DataFrame()
-    # Loop over rows
-    for chunk in tqdm(input_string.split(record_delimiter)[1:], disable=disable_progress_bar):
-        chunk_buffer: Dict[str, Any] = dict()
-        raw_breaks: List[str] = chunk.split(left)[1:]
-        middle: List[str] = [x.split(right)[0] for x in raw_breaks]
-        # Loop over possible tokens
-        for entry in middle:
-            if key_value_delimiter not in entry:
-                continue  # Do not attempt to parse if the delimiter is not present
-            substrings: List[str] = entry.split(key_value_delimiter)
-            key: str = substrings[0]
-            if not key:
-                continue  # Do not continue to parse if the kee is known
-            value: str = ''.join([x + key_value_delimiter for x in substrings[1:]])[:-1]
-            chunk_buffer.setdefault(key, value)  # Add the value to this row
-        if chunk_buffer:
-            df_output = pd.concat([df_output, pd.DataFrame(chunk_buffer, index=[-1])], ignore_index=True)
-    if 'dataframe' in return_type.lower():
-        return df_output
-    elif 'multiset' in return_type.lower():
-        return VectorMultiset(data=df_output)
-    elif 'sequence' in return_type.lower():
-        return VectorSequence(data=df_output, basis_col_name=basis_col_name)
-
-
-def auto_extract_from_file(file_path: Union[str, pathlib.Path], record_delimiter: str = '[@@@]',
-                           left: str = '[<<', key_value_delimiter: str = '=', right: str = '>>]',
-                           basis_col_name: str = None, return_type: str = 'dataframe',
-                           disable_progress_bar: bool = None) -> Union[pd.DataFrame, VectorSequence, VectorMultiset]:
-    """
-    Extracts a data frame from a file
-
-    Input: "the [@@@] quick [<<x='foo'>>] brown[<<y=9>>] [@@@]fox [<<y=93>>]"
-    Output: (data frame)
-                   x   y
-            0  'foo'   9
-            1    NaN  93
-
-    Hint, write your logs using python fstrings like f"Foo [<<{x=}>>] and [<<{y=}>>]"
-
-    :param file_path: file to read
-    :param record_delimiter: substring between rows
-    :param left: left side of a record
-    :param key_value_delimiter: marker between name and
-    :param right: right side of a record
-    :param basis_col_name: optional, specify a basis column name for a VectorSequence (otherwise ignored)
-    :param return_type: what format to return the data in
-    :param disable_progress_bar: pass anything True to silence the progress bar
-    :return: pandas.DataFrame or VectorMultiset or VectorSequence
-    """
-    with open(file_path, 'r') as f:
-        input_string: str = f.read()
-    return auto_extract_from_text(input_string=input_string, return_type=return_type, record_delimiter=record_delimiter,
-                                  left=left, key_value_delimiter=key_value_delimiter, right=right,
-                                  basis_col_name=basis_col_name, disable_progress_bar=disable_progress_bar)
-
-
-def extract_text_to_dataframe(input_string: str, tokens_dictionary: Dict[str, Tuple[str, str]],
-                              record_delimiter: str = '[@@@]', disable_progress_bar: bool = None) -> pd.DataFrame:
-    """
-    Extracts a pandas dataframe from a string
-
-    :param input_string: string (e.g. logs file) to be parsed
-    :param record_delimiter: The string that should be used to chunk up the string into observations / rows
-    :param tokens_dictionary: Extraction rules. Key = label for column, value = (before token, after token)
-    :param disable_progress_bar: pass anything True to silence the progress bar
-    :return: Vector data set extracted from the input string
-    """
-    df_output: pd.DataFrame = pd.DataFrame()
-    for chunk in tqdm(input_string.split(record_delimiter), disable=disable_progress_bar):
-        chunk_buffer: Dict[str, Any] = dict()
-        for key, (before_token, after_token) in tokens_dictionary.items():
-            if before_token in chunk:
-                target_chunk = chunk.split(before_token)[1]
-                if after_token in target_chunk:
-                    chunk_buffer.setdefault(key, target_chunk.split(after_token)[0])
-        if chunk_buffer:
-            df_output = pd.concat([df_output, pd.DataFrame(chunk_buffer, index=[-1])], ignore_index=True)
-    return df_output
-
-
-def extract_text_to_vector(input_string: str, tokens_dictionary: Dict[str, Tuple[str, str]],
-                           record_delimiter: str = '[@@@]', disable_progress_bar: bool = None,
-                           basis_col_name: str = None) -> Union[VectorMultiset, VectorSequence]:
-    """
-    Extracts a VectorMultiset from a string (or a VectorSequence if you specify `basis_col_name`)
-
-    :param input_string: string (e.g. logs file) to be parsed
-    :param record_delimiter: The string that should be used to chunk up the string into observations / rows
-    :param tokens_dictionary: Extraction rules. Key = label for column, value = (before token, after token)
-    :param basis_col_name: Optional - if specified returns a VectorSequence
-    :param disable_progress_bar: pass anything True to silence the progress bar
-    :return: Vector data set extracted from the input string
-    """
-    df_output: pd.DataFrame = extract_text_to_dataframe(input_string=input_string, record_delimiter=record_delimiter,
-                                                        tokens_dictionary=tokens_dictionary,
-                                                        disable_progress_bar=disable_progress_bar)
-    if basis_col_name:
-        return VectorSequence(data=df_output, basis_col_name=basis_col_name)
-    else:
-        return VectorMultiset(data=df_output)
-
-
-def extract_file_to_vector(file_path: Union[str, pathlib.Path], record_delimiter: str,
-                           tokens_dictionary: Dict[str, Tuple[str, str]], disable_progress_bar: bool = None,
-                           basis_col_name: str = None) -> Union[VectorSequence, VectorMultiset]:
-    """
-    Wrapper for extract_from_text that pulls vector data out of a file, such as raw log output
-
-    :param file_path: file to read
-    :param record_delimiter: The string that should be used to chunk up the string into observations / rows
-    :param tokens_dictionary: Extraction rules. Key = label for column, value = (before token, after token)
-    :param basis_col_name: Optional - if specified returns a VectorSequence
-    :param disable_progress_bar: pass anything True to silence the progress bar
-    :return: Vector data set extracted from the input string
-    """
-    with open(file_path, 'r') as f:
-        return extract_text_to_vector(input_string=f.read(), record_delimiter=record_delimiter,
-                                      tokens_dictionary=tokens_dictionary, basis_col_name=basis_col_name,
-                                      disable_progress_bar=disable_progress_bar)
-
-
 def correlation_matrix(dataframe: pd.DataFrame, use_cols: List[str] = None, exclude_cols: List[str] = None,
                        correlation_method: str = 'pearson', style: Style = None, **kwargs) -> pd.DataFrame:
     """
